﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Net.Http.Headers;

namespace ConsoleApplication1
{
    class ReportServerClient
    {
        static readonly MediaTypeFormatter defaultFormatter;
        public string BaseAddress { get; set; }
        public System.Net.Http.HttpClient client;
        public ReportServerClient(string uri)
        {
            this.client = new System.Net.Http.HttpClient();
            this.BaseAddress = uri;

            this.client.BaseAddress = new Uri(this.BaseAddress);
        }

        static ReportServerClient()
        {
            defaultFormatter = CreateFormatter();
        }

        static MediaTypeFormatter CreateFormatter()
        {
            var formatter = new JsonMediaTypeFormatter();

            var json = formatter.SerializerSettings;

            json.DateTimeZoneHandling = Newtonsoft.Json.DateTimeZoneHandling.Unspecified; // Preserves datetime values without timezone information.
            json.NullValueHandling = Newtonsoft.Json.NullValueHandling.Include;

            return formatter;
        }

        public void Login(string username, string password)
        {
            var headers =
                new List<KeyValuePair<string, string>>
                {
                    new KeyValuePair<string, string>("grant_type", "password"),
                    new KeyValuePair<string, string>("username", username),
                    new KeyValuePair<string, string>("password", password)
                };

            var content =
                new FormUrlEncodedContent(headers);

            var tokenTask = this.client.PostAsync("/Token", content);
            tokenTask.Wait();
            var response = tokenTask.Result;

            if (response.IsSuccessStatusCode)
            {
                var userTokenTask =
                    response.Content.ReadAsAsync<UserToken>();

                var userToken =
                    userTokenTask.Result;

                this.client.DefaultRequestHeaders.Authorization =
                    new AuthenticationHeaderValue("Bearer", userToken.AccessToken);
            }
            else
            {
                var error = response.Content.ReadAsAsync<ErrorModel>().Result;

                throw new Exception(error.Description);
            }
        }

        public IEnumerable<CategoryModel> GetCategories()
        {
            string route = this.BaseAddress + "categories";
            var categories = this.client.GetAsync(route).Result.Content.ReadAsAsync<List<CategoryModel>>();
            return categories.Result;
        }

        public string CreateReport(CreateReportInfoData reportData)
        {
            using (var content = new MultipartFormDataContent())
            {
                var properties = TypeDescriptor.GetProperties(reportData);

                foreach (PropertyDescriptor property in properties)
                {
                    var value = property.GetValue(reportData);

                    if (value is string)
                    {
                        content.Add(new StringContent((string)value), property.Name);
                    }
                }

                var fileContent =
                    new StreamContent(new MemoryStream(reportData.Content));

                fileContent.Headers.ContentType =
                    new MediaTypeHeaderValue("application/x-zip-compressed");

                fileContent.Headers.ContentDisposition =
                    new ContentDispositionHeaderValue("form-data")
                    {
                        Name = "ReportFile",
                        FileName = Path.ChangeExtension("report", reportData.Extension)
                    };

                content.Add(fileContent);

                string route = this.BaseAddress + "reports";

                var uploadTask = this.client.PostAsync(route, content);
                uploadTask.Wait();

                
                EnsureSuccessStatusCode(uploadTask.Result);

                return uploadTask.Result.Content.ReadAsStringAsync().Result;
            }
        }

        private static void EnsureSuccessStatusCode(HttpResponseMessage response)
        {
            if (!response.IsSuccessStatusCode)
            {
                throw
                    new Exception(
                        response.ReasonPhrase +
                        Environment.NewLine +
                        response.RequestMessage.RequestUri);
            }
        }        
    }
}
